<?php

/**
 * ServerSideGA class file.
 *
 * @author Tuan Phan <tphanx@gmail.com>
 * @package ssga
 * @version 1.0.1
 */

include_once __DIR__ . '/src/autoload.php';

use UnitedPrototype\GoogleAnalytics;

/**
 * ServerSideGA is an application component used for tracking page view/event/...
 * via Your website Google Analytic Account
 * 
 * You may configure it as below.  Check the public attributes and setter
 * methods of this class for more options.
 * 
 * <pre>
 * return array(
 * 	...
 * 	'components' => array(
 * 		'ssga' => array(
 * 			'class' => 'ext.ssga.ServerSideGA',
 * 			'gaId' => 'UA-12345678-9',
 * 		),
 * 		...
 * 	)
 * );
 * </pre>
 * 
 * Example usage:
 * <pre>
 * Yii::app()->ssga->trackPageView('/demo', 'Demo Page');
 * Yii::app()->ssga->trackEvent('Document', 'Download', 'demo-document.pdf');
 * </pre>
 */
class ServerSideGA  extends CApplicationComponent {
    
    public $gaId;
    public $domain;
    
    private $_tracker;
    private $_visitor;
    private $_session;
    
    public function init() {
        if(!$this->domain) {
            $this->domain = $_SERVER['HTTP_HOST'];
        }
        parent::init();
    }

    /**
	 * @return \UnitedPrototype\GoogleAnalytics\Tracker
	 */
    public function getTracker() {
        if(!$this->_tracker) {
            if(!$this->gaId) {
                throw new CException('gaId is required for Server Side GA');
            }
            // Initilize GA Tracker
            $this->_tracker = new GoogleAnalytics\Tracker($this->gaId, $this->domain);
        }
        return $this->_tracker;
    }
    
    /**
	 * @return \UnitedPrototype\GoogleAnalytics\Visitor
	 */
    public function getVisitor() {
        $session = Yii::app()->session;
        $this->_visitor = $session->get('_SSGA_VISITOR');
        if(!$this->_visitor) {
            // Assemble Visitor information
            $this->_visitor = new GoogleAnalytics\Visitor();
            $this->_visitor->fromServerVar($_SERVER);
            if(!empty($_COOKIE['__utma'])) {
                $this->_visitor->fromUtma($_COOKIE['__utma']);
            } else {
                $this->_visitor->setUniqueId($session->getSessionID());
            }
            //$this->_visitor->setScreenResolution('1024x768');
            $this->_visitor->addSession($this->getSession());
            $session->add('_SSGA_VISITOR', $this->_visitor);
        }
        return $this->_visitor;
    }
    
    /**
	 * @return \UnitedPrototype\GoogleAnalytics\Session
	 */
    public function getSession() {
        $session = Yii::app()->session;
        $this->_session = $session->get('_SSGA_SESSION');
        if(!$this->_session) {
            // Assemble Session information
            $this->_session = new GoogleAnalytics\Session();
            if(!empty($_COOKIE['__utmb'])) {
                $this->_session->fromUtmb($_COOKIE['__utmb']);
            }
            $session->add('_SSGA_SESSION', $this->_session);
        }
        return $this->_session;
    }

    /**
     * Track Google Analytic Page View.
     * @param string $url
     * @param string $title
     */
    public function trackPageView($url, $title) {
        // Assemble Page information
        $page = new GoogleAnalytics\Page($url);
        if(!empty($_SERVER['HTTP_REFERER'])) {
            $page->setReferrer($_SERVER['HTTP_REFERER']);
        }
        $page->setTitle($title);

        // Track page view
        $this->getTracker()->trackPageview($page, $this->session, $this->visitor);
    }
    
    /**
     * Track Google Analytic Event.
     * @param string $category
     * @param string $action
     * @param string $label
     */
    public function trackEvent($category, $action, $label) {
        // Assemble Event information
        $event = new GoogleAnalytics\Event($category, $action, $label);
        
        // Track event
        $this->getTracker()->trackEvent($event, $this->session, $this->visitor);
    }
}
